import '_model.dart';

class AssistantNotification {
  int? date;
  dynamic type;
  dynamic data;

  AssistantNotification({this.date, this.type, this.data});

  factory AssistantNotification.fromJson(map) {
    return AssistantNotification(
        date: map['date'], type: map['type'], data: map['data']);
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['date'] = date;
    data['type'] = type;
    data['data'] = this.data;
    return data;
  }
}

enum LoopModeRepetitive {
  every15min,
  every30min,
  every1h,
  every2h,
  every3h,
  every4h,
  every6h,
  every8h,
  every10h,
  every12h,
}

enum LoopModeDate { onlyOne, everyDay, everyWeek, everyMonth, everyYear }

class ReminderScheduled {
  static String repetitive = 'repetitive';
  static String date = 'date';

  int? id;
  ReminderType? reminderType;
  String? typeSchedule;
  String? comment;

  //repetitive
  int? loopModeRepetitive;
  List<int>? dayOfWeek;
  int? from;
  int? to;

  //date
  int? loopModeDate;
  int? datetime;

  ReminderScheduled({
    this.id,
    this.datetime,
    this.reminderType,
    this.comment,
    this.dayOfWeek,
    this.from,
    this.loopModeDate,
    this.loopModeRepetitive,
    this.to,
    this.typeSchedule,
  });

  ReminderScheduled.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    datetime = json['datetime'];
    if (json['reminderType'] != null) {
      reminderType = ReminderType.fromJson(json['reminderType']);
    }
    comment = json['comment'];
    dayOfWeek = json['dayOfWeek']?.map<int>((e) => e as int).toList();
    from = json['from'];
    loopModeDate = json['loopModeDate'];
    loopModeRepetitive = json['loopModeRepetitive'];
    to = json['to'];
    typeSchedule = json['typeSchedule'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['id'] = id;
    data['datetime'] = datetime;
    data['reminderType'] = reminderType?.toJson();
    data['comment'] = comment;
    data['dayOfWeek'] = dayOfWeek;
    data['from'] = from;
    data['loopModeDate'] = loopModeDate;
    data['loopModeRepetitive'] = loopModeRepetitive;
    data['to'] = to;
    data['typeSchedule'] = typeSchedule;
    return data;
  }
}

class CardConfiguration {
  late String degrees;
  late bool isDisplayWeather;
  late bool isDisplayHistory;

  CardConfiguration(
      {this.degrees = 'C',
      this.isDisplayWeather = true,
      this.isDisplayHistory = true});

  CardConfiguration.fromJson(Map<String, dynamic> json) {
    degrees = json['degrees'];
    isDisplayWeather = json['isDisplayWeather'];
    isDisplayHistory = json['isDisplayHistory'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['degrees'] = degrees;
    data['isDisplayWeather'] = isDisplayWeather;
    data['isDisplayHistory'] = isDisplayHistory;
    return data;
  }
}

class PersonalEventAndCalendarModel {
  bool? isMemory;
  bool? isReminderCard;
  int? cardType;
  dynamic calendarId;
  dynamic calendarEventId;
  String? personalEventId;
  String? localPathImage;
  String? color;
  ReminderScheduled? reminderScheduled;
  CalmModel? calmModel;

  PersonalEventAndCalendarModel({
    this.cardType,
    this.calendarId,
    this.calendarEventId,
    this.personalEventId,
    this.localPathImage,
    this.isMemory = false,
    this.isReminderCard = false,
    this.color,
    this.reminderScheduled,
    this.calmModel,
  });

  PersonalEventAndCalendarModel.fromJson(Map<String, dynamic> json) {
    color = json['color'];
    isMemory = json["isMemory"];
    isReminderCard = json["isReminderCard"];
    calendarId = json["calendarId"];
    calendarEventId = json["calendarEventId"];
    cardType = json["cardType"];
    if (json["personalEventId"] is String) {
      personalEventId = json["personalEventId"];
    }
    if (json["localPathImage"] is String) {
      localPathImage = json["localPathImage"];
    }
    if (json["reminderScheduled"] is Map) {
      reminderScheduled = ReminderScheduled.fromJson(json["reminderScheduled"]);
    }
    if (json["calmModel"] is Map) {
      calmModel = CalmModel.fromJson(json["calmModel"]);
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['color'] = color;
    data['isMemory'] = isMemory;
    data['isReminderCard'] = isReminderCard;
    data['calendarId'] = calendarId;
    data['calendarEventId'] = calendarEventId;
    data['personalEventId'] = personalEventId;
    data['localPathImage'] = localPathImage;
    data['cardType'] = cardType;
    data['reminderScheduled'] = reminderScheduled?.toJson();
    data['calmModel'] = calmModel?.toJson();
    return data;
  }
}

class EventLocalModel {
  dynamic message;
  int date;
  int cardType;
  int? duration;

  EventLocalModel(this.date, this.message, this.cardType, {this.duration});

  factory EventLocalModel.fromMap(map) {
    return EventLocalModel(map['date'], map['message'], map['cardType'],
        duration: map['duration']);
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['message'] = message;
    data['date'] = date;
    data['cardType'] = cardType;
    data['duration'] = duration;
    return data;
  }
}
