import '../resources.dart';

class HomeRepo {
  HomeRepo._();

  static HomeRepo? _instance;

  factory HomeRepo([AppApi? api]) {
    _instance ??= HomeRepo._();
    _instance!._api = api ?? AppApiImp();
    return _instance!;
  }

  late AppApi _api;

  Future<List<HomePlaceModel>> getHomesAroundMe(lat, lng,
      {distance = 100000, languageCode}) async {
    NetworkResponse response = await _api.getHomesAroundMe({
      'lat': lat,
      'lng': lng,
      'distance': distance,
      'languageCode': languageCode
    });
    return response.data ?? [];
  }

  Future<List<BrPromotedVideoModel>> getPromotedVideosForOoh({lat, lng}) async {
    NetworkResponse response = await _api.getPromotedVideosForOoh(lat, lng);
    return response.data ?? [];
  }

  Future<List<HomePurposeType>> getHomePurposeTypesList(params) async {
    NetworkResponse response = await _api.getHomePurposeTypesList(params);
    return response.data ?? [];
  }

  Future<List<HomelidoBlogPost>> getBlogPosts(params) async {
    NetworkResponse response = await _api.getBlogPosts(params);
    return response.data ?? [];
  }

  Future<List<LegalInfo>> getLegalContent(params) async {
    NetworkResponse response = await _api.getLegalContent(params);
    return response.data ?? [];
  }

  Future<List<HomeCountryAvaliable>> getCountriesAvailableForHomes(
      params) async {
    NetworkResponse response = await _api.getCountriesAvailableForHomes(params);
    return response.data ?? [];
  }

  Future<List<HomeCategory>> getHomeCatList(params) async {
    NetworkResponse response = await _api.getHomeCatList(params);
    return response.data ?? [];
  }

  Future<HomeDetailInfo?> getHomeDetails(params) async {
    NetworkResponse response = await _api.getHomeDetails(params);
    return response.data;
  }

  Future<HomeFiltersAvailable?> getCurrentFiltersAvailable(params) async {
    NetworkResponse response = await _api.getCurrentFiltersAvailable(params);
    return response.data;
  }

  Future<List<HomeCity>> getCitiesListByPurposeId(params) async {
    NetworkResponse response = await _api.getCitiesListByPurposeId(params);
    return response.data ?? [];
  }

  Future<List<HomeCity>> getHomeCitiesList(params) async {
    NetworkResponse response = await _api.getHomeCitiesList(params);
    return response.data ?? [];
  }

  Future<dynamic> getHomesListByFilters(params) async {
    NetworkResponse response = await _api.getHomesListByFilters(params);
    return response.data;
  }

  Future<bool> setHomeBookmark(params) async {
    NetworkResponse response = await _api.setHomeBookmark(params);
    return response.data ?? false;
  }
  
  Future<bool> updateActiveForHome(params) async {
    NetworkResponse response = await _api.updateActiveForHome(params);
    return response.data ?? false;
  }

  Future<bool> deleteHome(params) async {
    NetworkResponse response = await _api.deleteHome(params);
    return response.data ?? false;
  }

  Future<HomeDetailPictureUrl?> setMainPicForHome(bytes, path, homeId) async {
    NetworkResponse response =
        await _api.setMainPicForHome(bytes, path, homeId);
    return response.data;
  }

  Future<List<HomeDetailPictureUrl>> setGalleryPicForHome(
      bytes, path, homeId) async {
    NetworkResponse response =
        await _api.setGalleryPicForHome(bytes, path, homeId);
    return response.data ?? [];
  }

  Future<HomeDetailInfo?> createOohHome(params) async {
    NetworkResponse response = await _api.createOohHome(params);
    return response.data;
  }

  Future<HomeDetailInfo?> updateOohHome(params) async {
    NetworkResponse response = await _api.updateOohHome(params);
    return response.data;
  }

  Future<List<HomePurposeMedia>> getMediaForOoh(params) async {
    NetworkResponse response = await _api.getMediaForOoh(params);
    return response.data ?? [];
  }

  Future<List<HomeDetailInfo>> getHomesListForUserOrPro(params) async {
    NetworkResponse response = await _api.getHomesListForUserOrPro(params);
    return response.data ?? [];
  }

  Future<List<HomeDetailInfo>> getCurrentHomesOfTerminal(params) async {
    NetworkResponse response = await _api.getCurrentHomesOfTerminal(params);
    return response.data ?? [];
  }

  Future<List<HomeDetailInfo>> getHomesDetailsByListFromTerminal(params) async {
    NetworkResponse r = await _api.getHomesDetailsByListFromTerminal(params);
    return r.data ?? [];
  }

  Future<List<HomeDetailInfo>> getHomeBookmarkList(params) async {
    NetworkResponse response = await _api.getHomeBookmarkList(params);
    return response.data ?? [];
  }

  Future<List<HomeAgency>> getListOfAgencies(params) async {
    NetworkResponse response = await _api.getListOfAgencies(params);
    return response.data ?? [];
  }

  Future<CurrencyOfCountry?> getCurrencyOfCountry(params) async {
    NetworkResponse response = await _api.getCurrencyOfCountry(params);
    return response.data;
  }

  Future<bool> removeGalleryPicForHome(params) async {
    NetworkResponse response = await _api.removeGalleryPicForHome(params);
    return response.data ?? false;
  } 

  Future<HomeDetailPictureUrl?> setLogoForAgency(bytes, path, homeId) async {
    NetworkResponse response = await _api.setLogoForAgency(bytes, path, homeId);
    return response.data;
  }

  Future<bool> updateAgency(params) async {
    NetworkResponse response = await _api.updateAgency(params);
    return response.data ?? false;
  }
}
